<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class LogTracking
{
    private $logTrackingService;

   public function __construct (\App\Service\LogTracking $logTrackingService,
                               ) {

      $this->logTrackingService = $logTrackingService;
    }

    public function errorLogs(Request $request, Response $response)
    {
        try {
            $params = $request->getQueryParams();
            
            $cdFilial = $this->logTrackingService->getParamsValue($params, 'CDFILIAL');
            $cdIntegracao = $this->logTrackingService->getParamsValue($params, 'CDINTEGRACAO');
            $order =  $this->logTrackingService->getOrder($params);
            $dtInclusao = $this->logTrackingService->ensureDTInclusaoExists($params);
            $search = (string) ($params['search'] ?? '');
            $paramsLog = [
                'cdFilial'          => $cdFilial,
                'cdIntegracao'      => $cdIntegracao,
                'dtInclusao'        => $dtInclusao,
                'search'            => $search,
                'order'            => $order,
            ];

            $this->logTrackingService->checkOldLogs();
            
            $datalogs = $this->logTrackingService->errorLogs($paramsLog);
            
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $datalogs,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($datalogs),
                    'page' => $params['page'] ?? 1
                ]
            ]));
    
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function deleteLogs(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $CDINTEGRACAOERR = !empty($params['CDINTEGRACAOERR']) ? $params['CDINTEGRACAOERR'] : null;

            if (empty($CDINTEGRACAOERR)) {
                throw new \Exception("Código do log não informado.");
            }

            $this->logTrackingService->deleteLogs($CDINTEGRACAOERR);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => 'Log excluído com sucesso',
                'data' => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function selectTableInteg(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $dataTableInteg = $this->logTrackingService->selectTableInteg();

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataTableInteg,
                'pagination' => [
                    'limit' => $params['limit'] ?? 5,
                    'total' => count($dataTableInteg),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function selectTableFilial(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $dataTableFilial = $this->logTrackingService->selectTableFilial();

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataTableFilial,
                'pagination' => [
                    'limit' => $params['limit'] ?? 5,
                    'total' => count($dataTableFilial),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getLogDetails(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $cdIntegracaoErr = !empty($params['CDINTEGRACAOERR']) ? $params['CDINTEGRACAOERR'] : null;

            if (empty($cdIntegracaoErr)) {
                throw new \Exception("Código do log não informado.");
            }

            $dataLogDetails = $this->logTrackingService->getLogDetails($cdIntegracaoErr);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataLogDetails
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }
}
