<?php

namespace App\Service;
use \Datetime;

class Dashboard {

    private $dashboardFactory;
    private $util;
    private $dateUtil;

    public function __construct(\App\Factory\Dashboard $dashboardFactory, \App\Util\Util $util, \App\Util\DateUtil $dateUtil) 
    {
        $this->dashboardFactory = $dashboardFactory;
        $this->util = $util;
        $this->dateUtil = $dateUtil;
    }

    public function dashboard($dtInicAberMesa, $dtFimAberMesa)
    {
        try {
            $dashboard = $this->dashboardFactory->dashboard($dtInicAberMesa, $dtFimAberMesa);

            $dataset = [
                'lastWeekDatasource' => [],
                'seriesInnerData' => [],
                'seriesOuterData' => [],
                'salesHour' => array_fill(0, 24, 0),
                'quantSalesToday' => 0,
                'valueSalesToday' => 0,
                'quantCancelToday' => 0,
                'valueCancelToday' => 0,
                'quantCancelCurrentMonth' => 0,
                'valueCancelCurrentMonth' => 0,
                'quantCancelLastMonth' => 0,
                'valueCancelLastMonth' => 0,
                'quantSalesCurrentMonth' => 0,
                'valueSalesCurrentMonth' => 0,
                'quantSalesLastMonth' => 0,
                'valueSalesLastMonth' => 0,
                'salesMonthModality' => [],
                'salesMonthModalityLegend' => [],
                'salesMonthModalityDatasource' => [],
                'salesMonthModalityData' => [],
                'salesMonthOrigin' => [],
                'salesMonthOriginLegend' => [],
                'salesMonthOriginDatasource' => [],
                'salesMonthOriginData' => [],
                'cancelMonthOrigin' => [],
                'cancelMonthOriginLegend' => [],
                'cancelMonthOriginDatasource' => [],
                'cancelMonthOriginData' => [],
                'quantSalesLast6Month' => 0,
                'quantCancelLast6Month' => 0,
                'valueSalesLast6Month' => 0,
                'valueCancelLast6Month' => 0,
            ];

            $quantSalesLastWeek = [];
            $quantCancelLastWeek = [];
            $valueSalesLastWeek = [];
            $todayInnerData = [];
            $todayOuterData = [];
            $foundNonTodayOrder = false;
            $foundNonCurrentWeekOrder = false;
            $foundNonCurrentMonthOrder = false;
            $foundNonPastMonthOrder = false;

            foreach ($dashboard as $row) {
                if ($row['IDSTCOMANDA'] === 'A') {
                    continue;
                }

                $row['IDORGCMDVENDA'] = $this->util->commandSourceDictionary($row['IDORGCMDVENDA']);
            
                if ($row['IDSTCOMANDA'] !== '4') {
                    $dataset['quantSalesLast6Month']++;
                    $dataset['valueSalesLast6Month'] += $row['VRPEDIDO'];

                    $this->processMonthlySalesData($row, $dataset, 'salesMonthModality', 'TOGO');
                    $this->processMonthlySalesData($row, $dataset, 'salesMonthOrigin', 'IDORGCMDVENDA');

                    if (!$foundNonTodayOrder && $this->dateUtil->isToday($row['DTHRABERMESA'])) {
                        $todayInnerData[$row['TOGO']] = ($todayInnerData[$row['TOGO']] ?? 0) + 1;
                        $todayOuterData[$row['IDORGCMDVENDA']] = ($todayOuterData[$row['IDORGCMDVENDA']] ?? 0) + 1;

                        $saleHour = ltrim(DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA'])->format('H'), '0');
                        $dataset['salesHour'][$saleHour] += 1;

                        $dataset['quantSalesToday'] += 1;
                        $dataset['valueSalesToday'] += $row['VRPEDIDO'];

                        $dataset['quantSalesCurrentMonth'] += 1;
                        $dataset['valueSalesCurrentMonth'] += $row['VRPEDIDO'];
                    } else {
                        $foundNonTodayOrder = true;
                        if (!$foundNonCurrentMonthOrder && $this->dateUtil->isCurrentMonth($row['DTHRABERMESA'])) {
                            $dataset['quantSalesCurrentMonth'] += 1;
                            $dataset['valueSalesCurrentMonth'] += $row['VRPEDIDO'];
                        } else {
                            $foundNonCurrentMonthOrder = true;

                            if (!$foundNonPastMonthOrder && $this->dateUtil->isLastMonth($row['DTHRABERMESA'])) {
                                $dataset['quantSalesLastMonth'] += 1;
                                $dataset['valueSalesLastMonth'] += $row['VRPEDIDO'];
                            }
                        }
                        if (!$foundNonCurrentWeekOrder && $this->dateUtil->isLastWeek($row['DTHRABERMESA'])) {
                            $day = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA'])->format('d/m');
        
                            $quantSalesLastWeek[$day] = ($quantSalesLastWeek[$day] ?? 0) + 1;
                            $valueSalesLastWeek[$day] = ($valueSalesLastWeek[$day] ?? 0) + $row['VRPEDIDO'];
                        } else {
                            $foundNonCurrentWeekOrder = true;
                        }
                    }
                } else {
                    $dataset['quantCancelLast6Month']++;
                    $dataset['valueCancelLast6Month'] += $row['VRPEDIDO'];

                    $this->processMonthlySalesData($row, $dataset, 'cancelMonthOrigin', 'IDORGCMDVENDA');
                    
                    if (!$foundNonTodayOrder && $this->dateUtil->isToday($row['DTHRABERMESA'])) {
                        $dataset['quantCancelToday'] += 1;
                        $dataset['valueCancelToday'] += $row['VRPEDIDO'];
                        $dataset['quantCancelCurrentMonth'] += 1;
                        $dataset['valueCancelCurrentMonth'] += $row['VRPEDIDO'];
                    } else {
                        $foundNonTodayOrder = true;
                        if (!$foundNonCurrentMonthOrder && $this->dateUtil->isCurrentMonth($row['DTHRABERMESA'])) {
                            $dataset['quantCancelCurrentMonth'] += 1;
                            $dataset['valueCancelCurrentMonth'] += $row['VRPEDIDO'];
                        } else {
                            $foundNonCurrentMonthOrder = true;

                            if (!$foundNonPastMonthOrder && $this->dateUtil->isLastMonth($row['DTHRABERMESA'])) {
                                $dataset['quantCancelLastMonth'] += 1;
                                $dataset['valueCancelLastMonth'] += $row['VRPEDIDO'];
                            }
                        }

                        if (!$foundNonCurrentWeekOrder && $this->dateUtil->isLastWeek($row['DTHRABERMESA'])) {
                            $day = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA'])->format('d/m');
        
                            $quantCancelLastWeek[$day] = ($quantCancelLastWeek[$day] ?? 0) + 1;
                        } else {
                            $foundNonCurrentWeekOrder = true;
                        }
                    }
                }
            }

            $this->getDatasource($dataset, 'salesMonthModality');
            $this->getDatasource($dataset, 'salesMonthOrigin');
            $this->getDatasource($dataset, 'cancelMonthOrigin'); 

            foreach ($todayInnerData as $modality => $quant) {
                $dataset['seriesInnerData'][] = [$modality . ' (' . $quant . ')', $quant];
            }

            foreach ($todayOuterData as $origin => $quant) {
                $dataset['seriesOuterData'][] = [$origin . ' (' . $quant . ')', $quant];
            }

            $day = new DateTime('yesterday');
            $weekDays = 7;

            for ($i=0; $i<$weekDays; $i++) {
                $date = $day->format('d/m');
                $dataset['lastWeekDatasource'][$i] = [$date, $quantCancelLastWeek[$date] ?? 0, $quantSalesLastWeek[$date] ?? 0, number_format($valueSalesLastWeek[$date] ?? 0, 2, '.', '')];
                $day->modify("-1 day");    
            }

            return $dataset;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function processMonthlySalesData($row, &$dataset, $name, $type) 
    {
        $data = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA']);
        $month = $data->format('n');

        if (!isset($dataset[$name][$month])) {
            $dataset[$name][$month] = [];
        }

        $modality = $row[$type];

        if (!isset($dataset[$name][$month][$modality])) {
            $dataset[$name][$month][$modality] = 0;
        }

        if (!in_array($row[$type], $dataset[$name.'Legend'])) {
            $dataset[$name.'Legend'][] = $row[$type];
            $newSeries = [
                'name' => $row[$type],
                'type' => 'line',
                'data' => array_fill(0, 12, 0),
                'label' => [
                    'show' => true,
                    'position' => 'top',
                    'color' => 'black',
                    'fontSize' => 12,
                    'formatter' => function($d) {
                        return $d['data'];
                    },
                ],
            ];
            $dataset[$name.'Datasource'][] = $newSeries;
        }

        $dataset[$name][$month][$modality] += 1;
    }

    private function getDatasource(&$dataset, $name) 
    {
        $retroMonths = 6;
        $dataset[$name.'Data'] = $this->dateUtil->getPreviousMonths($retroMonths, true);
        $todayMonth = date('n');
        $nextMonth = new DateTime();
        $nextMonth->setDate($nextMonth->format('Y'), $nextMonth->format('m'), 1);
        $nextMonth->modify('-' . $retroMonths . ' months');
        $k = 0;

        while (intval($nextMonth->format('n')) !== intval($todayMonth + 1)) {
            $j = 0;
            $totalSalesMonth = 0;
            for ($j; $j < count($dataset[$name.'Legend']); $j++) {
                if (isset($dataset[$name][$nextMonth->format('n')])) {
                    $dataset[$name.'Datasource'][$j]['data'][$k] = $dataset[$name][$nextMonth->format('n')][$dataset[$name.'Legend'][$j]] ?? 0;
                    $totalSalesMonth += $dataset[$name][$nextMonth->format('n')][$dataset[$name.'Legend'][$j]] ?? 0;
                }
            }

            $dataset[$name.'Data'][$k] = $dataset[$name.'Data'][$k] . '(' . $totalSalesMonth . ')';
            $nextMonth->modify('+1 month');
            $k += 1;
        }
    }
}